// -------------------------------------------------------------------------------------------------------------
// ------------      Funktionen zur Versendung einer SMS ber ein Siemens Handy (M35)          -----------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------------------------
// Datenformat der SMS: PDU
// -------------------------------------------------------------------------------------------------------------

#include <avr/interrupt.h>

// Definition Maximalwerte
#define SMS_MESSAGE_MAX_LEN 160
#define SMS_TELNR_MAX_LEN 15

// Variablen erstellen
unsigned char uart0_empfang[100] = {0};
int uart0_cnt = 0;

// Prototyp
void wait_between_sms(void);    // Wartefunktion, um mehrere SMS nacheinander senden zu knnen



/**
* Wandelt ein Zeichen in eine Hexzahl um
* @param data Zeichen, welches umgewandelt werden soll
* @return Hexzahl equivalent von 'data'
*/
uint8_t str2int(char data){
  data -= '0';
  if (data>9) data-= 7;
  return data;
}


/**
* Initialisiert die serielle Schnittstelle und ffnet einen Kanal fr Standard Ein-/Ausgabe (printf, gets, scanf,...)
*/
void sms_init(void){

	uart_puts("ATE0");			// Echo off, reduce traffic
	uart_putc(13);
	_delay_ms(200);
	
}


/**
* Sendet eine SMS mit dem Inhalt "nachricht" an "zielnr"
* @param zielnr Pointer auf Zielnummer der SMS
* @param nachricht Pointer auf Nachricht der SMS an Zielnummer
*/
void sms_send(char* zielnr, char* nachricht)
{
	_delay_ms(500);
	_delay_ms(500);
	_delay_ms(500);
	_delay_ms(500);
		
	uint8_t cnt_shiftzero=0;
	uint8_t cnt_nr=0;					// Zhler fr Zielnummer, nach Umwandlung = Lnge der Zielnummer
	uint8_t cnt_txt=0;					// Zhler fr Nachricht, nach Umwandlung = Lnge der Nachricht
	uint8_t shift=0;
	uint8_t i=0;
	char nachricht_neu[SMS_MESSAGE_MAX_LEN+1]={"\0"};
	char zielnr_neu[SMS_TELNR_MAX_LEN+1]={" "};

	
	// Nachricht umwandeln in PDU-Format
	while (nachricht[cnt_txt]!=0)
	{
		if (shift==7) {
			shift=0;
			cnt_txt++;
			cnt_shiftzero++;
		}
		else {								
			nachricht_neu[cnt_txt-cnt_shiftzero] = (((nachricht[cnt_txt]&0x7F)>>shift) | ((nachricht[cnt_txt+1])<<(7-shift)));
			shift++;
			cnt_txt++;
		}
	}
	nachricht_neu[cnt_txt-cnt_shiftzero]=nachricht[cnt_txt];	// '0' anfgen , Endestring
	
	// Zielnummer neu zusammensetzen
	// Bsp. 01 62 64 94 30 0 --> 10 26 46 49 03 F0
	// wenn 2 Bytes nicht voll ausgenutzt werden, 'F' einfgen
	while (zielnr[cnt_nr]!=0) {
		if (zielnr[cnt_nr+1]==0) {
			zielnr_neu[cnt_nr]='F';	
			zielnr_neu[cnt_nr+1]=zielnr[cnt_nr];
			cnt_nr++;
			cnt_nr++;
			zielnr_neu[cnt_nr]=zielnr[cnt_nr-1];	// '0' anfgen , Endestring
			break;
		}
		else {
			zielnr_neu[cnt_nr]=zielnr[cnt_nr+1];
			zielnr_neu[cnt_nr+1]=zielnr[cnt_nr];
			cnt_nr=cnt_nr+2;
		}
	}

	//Anzahl der Zeichen im PDU-String/2-->Hex Paare -2 (ersten 2 Hex Zeichen weglassen)
	//-->'Anzahl der Nachrichtenzeichen' + 'Anzahl der Nummern /2' (/2 da nur Hex Paare gezhlt werden) + '7' (zustzliche Header)
	if (zielnr_neu[cnt_nr-2]=='F') {
		uart_puts("AT+CMGS=");
		char adbuffer4[30];
		sprintf(adbuffer4,"%02i",(cnt_txt-cnt_shiftzero+(cnt_nr/2)+7));
		uart_puts(adbuffer4);
		uart_putc(13);
		cnt_nr--;		//Lnge der Zielnummer -1, da 'F' angefgt
	}
	else {
		uart_puts("AT+CMGS=");
		char adbuffer4[30];
		sprintf(adbuffer4,"%02i",(cnt_txt-cnt_shiftzero+(cnt_nr/2)+7));
		uart_puts(adbuffer4);
		uart_putc(13);
	}
	
	// hier ggf. auf Antwort von Handy warten - hat aber noch nie rchtig funktioniert
	char adbuffer[30];
	_delay_ms(100);
	uart_puts("000100");
	sprintf(adbuffer,"%02X",cnt_nr);
	uart_puts(adbuffer);
	uart_puts("81");
	
	uart_puts((zielnr_neu));
	uart_puts("0000");
	char adbuffer2[30];
	sprintf(adbuffer2,"%02X",cnt_txt);
	uart_puts(adbuffer2);
	
	
	for (i = 0;i<(cnt_txt-cnt_shiftzero);i++){
		char adbuffer3[30]={0};
		sprintf(adbuffer3,"%02X",nachricht_neu[i]);
		uart_puts(adbuffer3);
	};

	uart_putc(26);
	_delay_ms(100);
	
	// 6s warten
	wait_between_sms();
}



// Zeigt Meldung auf Handy Display an
void sms_display_text(char* nachricht)
{	
	uint8_t i;
	uint8_t cnt_txt=0;
	
	uint8_t cnt_shiftzero=0;
	uint8_t shift=0;
	char nachricht_neu[SMS_MESSAGE_MAX_LEN+1]={"\0"};
	
	while (nachricht[cnt_txt]!=0)
	{
		if (shift==7) {
			shift=0;
			cnt_txt++;
			cnt_shiftzero++;
		}
		else {								
			nachricht_neu[cnt_txt-cnt_shiftzero] = (((nachricht[cnt_txt]&0x7F)>>shift) | ((nachricht[cnt_txt+1])<<(7-shift)));
			shift++;
			cnt_txt++;
		}
	}
	nachricht_neu[cnt_txt-cnt_shiftzero]=nachricht[cnt_txt];	// '0' anfgen , Endestring
    
	char adbuffer10 [25] = {0};
	
	sprintf(adbuffer10,"AT^SSTK=%02i,0",(cnt_txt+13+1));	// 13 Header, 1 't' vor nachricht
	uart_puts(adbuffer10);
	uart_putc(13);
	
	_delay_ms(200);

	char adbuffer11[5] = {0};
	sprintf(adbuffer11,"D0%02X",(cnt_txt+13+1-2));	// Lnge des PDU String - 1
	uart_puts(adbuffer11);
	/*
	01	--> Commad Detail Tag
	03	--> Lnge
	01	--> Command Number
	21	--> Type of Command (Display Text)
	80	--> Command Qualifier (Display TExt, wait for user clear message) - 0 fr auto lschen
	02	--> Device Identity Tag
	02	--> Lnge
	81	--> Source Device Identity (SIM)
	02	--> Destination Device Identity (Display)
	0D	--> Text String Tag
	*/
	uart_puts("0103012180020281020D");	//Header
	char adbuffer12[5]= {0};
	sprintf(adbuffer12,"%02X",(cnt_txt+1));	// Lnge des Text String
	uart_puts(adbuffer12);
	
	char adbuffer13[5]= {0};
	sprintf(adbuffer13,"%02X",'t');			// 't' vor nachricht einfgen
	uart_puts(adbuffer13);
	
	// Text String in Hex umwandeln
	for ( i=0 ; i<cnt_txt ; i++ ){
		char adbuffer14[10]= {0};
		sprintf(adbuffer14,"%02X",(nachricht_neu[i]));
		uart_puts(adbuffer14);
	}
	// ENDE Zeichen, strg+z
	uart_putc(26);

	_delay_ms(200);		// auf Antwort von Mobiltelefon warten	
}


// Wartefunktion um mehrere SMS hintereinander senden zu knnen
void wait_between_sms(void){
	
	// wartet mittels Timer1 
	TCCR1B &= 0b11111000;                 // Timer 1 stoppen
	TIMSK1 &= ~(0b00000010);			  // Timer1 Interrupt aus
        
    unsigned long int ovfcounter = 0;
    
    TCCR2A &= 0b00000000;
    TCCR2B &= 0b11111000;
    
    TCNT2 = 0;                  // Counterregister leeren
    TIMSK2 = 0;
	if(TIFR2&1)TIFR2|=1;        // Bei Bedarf Overflow Flag clear
    
    do{
        TCCR2B |=0b00000111;     // Timer starten mit Prescaler 1024
		while(TCNT2<250);
        TCCR2B &= 0b11111000;             // Timer stoppen
		if(TIFR2&1)TIFR2|=1;        // Bei Bedarf Overflow Flag clear
        TIFR2|=1;               // OVF Flag clear
        TCNT2 = 0;              // Counter Register leeren
        ovfcounter++;           // OVF Counter erhoehen
		
		
		
    }while(ovfcounter<400);
	
	
}
